@@ -4,7 +4,10 @@ import enum
 import math
 import numpy
 import logging
-import collections
+try:  # pragma: no cover
+    from collections import abc
+except ImportError:  # pragma: no cover
+    import collections as abc
 
 from python_utils import logger
 
@@ -44,10 +47,10 @@ class RemoveDuplicates(enum.Enum):
 
     @classmethod
     def map(cls, value):
-        if value and value in cls:
-            pass
-        elif value:
+        if value is True:
             value = cls.SINGLE
+        elif value and value in cls:
+            pass
         else:
             value = cls.NONE
 
@@ -74,7 +77,7 @@ def logged(class_):
 
 
 @logged
-class BaseMesh(logger.Logged, collections.Mapping):
+class BaseMesh(logger.Logged, abc.Mapping):
     '''
     Mesh object with easy access to the vectors through v0, v1 and v2.
     The normals, areas, min, max and units are calculated automatically.
@@ -105,24 +108,33 @@ class BaseMesh(logger.Logged, collections.Mapping):
     >>> mesh.v1[0] += 2
 
     >>> # Check item 0 (contains v0, v1 and v2)
-    >>> mesh[0]
-    array([ 1.,  1.,  1.,  2.,  2.,  2.,  0.,  0.,  0.], dtype=float32)
-    >>> mesh.vectors[0] # doctest: +NORMALIZE_WHITESPACE
-    array([[ 1.,  1.,  1.],
-           [ 2.,  2.,  2.],
-           [ 0.,  0.,  0.]], dtype=float32)
-    >>> mesh.v0[0]
-    array([ 1.,  1.,  1.], dtype=float32)
-    >>> mesh.points[0]
-    array([ 1.,  1.,  1.,  2.,  2.,  2.,  0.,  0.,  0.], dtype=float32)
-    >>> mesh.data[0] # doctest: +NORMALIZE_WHITESPACE
-    ([ 0.,  0.,  0.], [[ 1.,  1.,  1.], [ 2.,  2.,  2.], [ 0.,  0.,  0.]], [0])
-    >>> mesh.x[0]
-    array([ 1.,  2.,  0.], dtype=float32)
+    >>> assert numpy.array_equal(
+    ...     mesh[0],
+    ...     numpy.array([1., 1., 1., 2., 2., 2., 0., 0., 0.]))
+    >>> assert numpy.array_equal(
+    ... mesh.vectors[0],
+    ... numpy.array([[1., 1., 1.],
+    ...     [2., 2., 2.],
+    ...     [0., 0., 0.]]))
+    >>> assert numpy.array_equal(
+    ...     mesh.v0[0],
+    ...     numpy.array([1., 1., 1.]))
+    >>> assert numpy.array_equal(
+    ...     mesh.points[0],
+    ...     numpy.array([1., 1., 1., 2., 2., 2., 0., 0., 0.]))
+    >>> assert numpy.array_equal(
+    ...     mesh.data[0],
+    ...     numpy.array((
+    ...             [0., 0., 0.],
+    ...             [[1., 1., 1.], [2., 2., 2.], [0., 0., 0.]],
+    ...             [0]),
+    ...         dtype=BaseMesh.dtype))
+    >>> assert numpy.array_equal(mesh.x[0], numpy.array([1., 2., 0.]))
 
     >>> mesh[0] = 3
-    >>> mesh[0]
-    array([ 3.,  3.,  3.,  3.,  3.,  3.,  3.,  3.,  3.], dtype=float32)
+    >>> assert numpy.array_equal(
+    ...     mesh[0],
+    ...     numpy.array([3., 3., 3., 3., 3., 3., 3., 3., 3.]))
 
     >>> len(mesh) == len(list(mesh))
     True
@@ -301,9 +313,18 @@ class BaseMesh(logger.Logged, collections.Mapping):
         squared_areas = (normals ** 2).sum(axis=1)
         return data[squared_areas > AREA_SIZE_THRESHOLD ** 2]
 
-    def update_normals(self):
-        '''Update the normals for all points'''
-        self.normals[:] = numpy.cross(self.v1 - self.v0, self.v2 - self.v0)
+    def update_normals(self, update_areas=True):
+        '''Update the normals and areas for all points'''
+        normals = numpy.cross(self.v1 - self.v0, self.v2 - self.v0)
+
+        if update_areas:
+            self.update_areas(normals)
+
+        normal = numpy.linalg.norm(normals, axis=1)
+        non_zero = normal > 0
+        if non_zero.any():
+            normals[non_zero] /= normal[non_zero][:, None]
+        self.normals[:] = normals
 
     def update_min(self):
         self._min = self.vectors.min(axis=(0, 1))
@@ -311,10 +332,29 @@ class BaseMesh(logger.Logged, collections.Mapping):
     def update_max(self):
         self._max = self.vectors.max(axis=(0, 1))
 
-    def update_areas(self):
-        areas = .5 * numpy.sqrt((self.normals ** 2).sum(axis=1))
+    def update_areas(self, normals=None):
+        if normals is None:
+            normals = numpy.cross(self.v1 - self.v0, self.v2 - self.v0)
+
+        areas = .5 * numpy.sqrt((normals ** 2).sum(axis=1))
         self.areas = areas.reshape((areas.size, 1))
 
+    def check(self):
+        '''Check the mesh is valid or not'''
+        return self.is_closed()
+
+    def is_closed(self):  # pragma: no cover
+        """Check the mesh is closed or not"""
+        if (self.normals.sum(axis=0) >= 1e-4).any():
+            self.warning('''
+            Your mesh is not closed, the mass methods will not function
+            correctly on this mesh.  For more info:
+            https://github.com/WoLpH/numpy-stl/issues/69
+            '''.strip())
+            return False
+        else:
+            return True
+
     def get_mass_properties(self):
         '''
         Evaluate and return a tuple with the following elements:
@@ -325,6 +365,8 @@ class BaseMesh(logger.Logged, collections.Mapping):
         Documentation can be found here:
         http://www.geometrictools.com/Documentation/PolyhedralMassProperties.pdf
         '''
+        self.check()
+
         def subexpression(x):
             w0, w1, w2 = x[:, 0], x[:, 1], x[:, 2]
             temp0 = w0 + w1
@@ -402,7 +444,7 @@ class BaseMesh(logger.Logged, collections.Mapping):
         axis = numpy.asarray(axis)
         # No need to rotate if there is no actual rotation
         if not axis.any():
-            return numpy.zeros((3, 3))
+            return numpy.identity(3)
 
         theta = 0.5 * numpy.asarray(theta)
 
@@ -421,7 +463,7 @@ class BaseMesh(logger.Logged, collections.Mapping):
                             [2 * (bc - ad), aa + cc - bb - dd, 2 * (cd + ab)],
                             [2 * (bd + ac), 2 * (cd - ab), aa + dd - bb - cc]])
 
-    def rotate(self, axis, theta, point=None):
+    def rotate(self, axis, theta=0, point=None):
         '''
         Rotate the matrix over the given axis by the given theta (angle)
 
@@ -442,6 +484,14 @@ class BaseMesh(logger.Logged, collections.Mapping):
         if not theta:
             return
 
+        self.rotate_using_matrix(self.rotation_matrix(axis, theta), point)
+
+    def rotate_using_matrix(self, rotation_matrix, point=None):
+        identity = numpy.identity(rotation_matrix.shape[0])
+        # No need to rotate if there is no actual rotation
+        if not rotation_matrix.any() or (identity == rotation_matrix).all():
+            return
+
         if isinstance(point, (numpy.ndarray, list, tuple)) and len(point) == 3:
             point = numpy.asarray(point)
         elif point is None:
@@ -451,12 +501,6 @@ class BaseMesh(logger.Logged, collections.Mapping):
         else:
             raise TypeError('Incorrect type for point', point)
 
-        rotation_matrix = self.rotation_matrix(axis, theta)
-
-        # No need to rotate if there is no actual rotation
-        if not rotation_matrix.any():
-            return
-
         def _rotate(matrix):
             if point.any():
                 # Translate while rotating
@@ -465,6 +509,10 @@ class BaseMesh(logger.Logged, collections.Mapping):
                 # Simply apply the rotation
                 return matrix.dot(rotation_matrix)
 
+        # Rotate the normals
+        self.normals[:] = _rotate(self.normals[:])
+
+        # Rotate the vectors
         for i in range(3):
             self.vectors[:, i] = _rotate(self.vectors[:, i])
 
